#!/bin/sh

# Verify and setup prepackaged components
# 1 - Return Result
# 2 - Source
# 3 - File Name
# 4 - Destination
# 5 - Package Type
verifyAndSetupComponent()
{
    SETUP_RESULT="$1"
    SOURCE_DIR="$2"
    SOURCE_FILE="$3"
    DESTINATION_DIR="$4"
    PKG_TYPE="$5"

    CURRENT_DIR=`pwd`
    if [ "$PKG_TYPE" = "jvm" ] ; then
        TARGET_DIR="$DESTINATION_DIR/jvm"
    else
        TARGET_DIR="$DESTINATION_DIR"
    fi

    if [ -f "$SOURCE_DIR/$SOURCE_FILE" ]; then
        if [ ! -d "$TARGET_DIR" ]; then
            mkdir "$TARGET_DIR"
        fi
        echo "Untar $SOURCE_FILE to $TARGET_DIR"
        if [ "$MY_PLATFORM" = "solaris" ] ; then
            cd "$TARGET_DIR"
            gzip -cdf "$SOURCE_DIR/$SOURCE_FILE" | /usr/sfw/bin/gtar xf - > /dev/null
            cd "$CURRENT_DIR"
        elif [ "$MY_PLATFORM" = "others" ] ; then
            tar -xzf "$SOURCE_DIR/$SOURCE_FILE" -C "$TARGET_DIR" > /dev/null
        fi
        eval $SETUP_RESULT="Y"
    else
        echo "$SOURCE_FILE not exists"
        eval $SETUP_RESULT="N"
    fi
}

# Prepare prepackaged components
# 1 - Return Result
preparePackagedComponents()
{
    PREPARE_RESULT="$1"
    eval $PREPARE_RESULT="N"

    RESULT_JVM_HOME="$2"
    eval $RESULT_JVM_HOME="Y"

    # Prepare jvm
    PKG_JVM_FILE=""
    case "`uname`" in
        Linux*)
            if [ "$MY_ARCH" = "64" ]; then
                PKG_JVM_FILE="jre-std-linux-amd64.tar.gz"
            elif [ "$MY_ARCH" = "32" ]; then
                PKG_JVM_FILE="jre-std-linux-x86-586.tar.gz"
            fi
            ;;
        *)
            ;;
    esac
    if [ -z "$PKG_JVM_FILE" ]; then
        echo "No JVM package is defined"
        eval $RESULT_JVM_HOME="N"
    else
        for word in $PKG_JVM_FILE
        do
            verifyAndSetupComponent SETUP_RESULT "$PKG_DIR" "$word" "$APP_TMP" "jvm"
            if [ "$SETUP_RESULT" = "N" ] ; then
                echo "Failed to prepare JVM files."
                return
            fi
        done
    fi

    # Prepare other components
    PKG_COMPONENT_FILE=""
    if [ "$MY_ARCH" = "64" ]; then
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-x64.tar.gz app-nix-cbp.tar.gz aua-common.tar.gz aua-native-nix-x64.tar.gz aua-nix-cbp.tar.gz util-common.tar.gz util-nix-cbp.tar.gz properties-common.tar.gz app-inst-nix-cbp.tar.gz aua-inst-nix-cbp.tar.gz"
    elif [ "$MY_ARCH" = "32" ]; then
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-x86.tar.gz app-nix-cbp.tar.gz aua-common.tar.gz aua-native-nix-x86.tar.gz aua-nix-cbp.tar.gz util-common.tar.gz util-nix-cbp.tar.gz properties-common.tar.gz app-inst-nix-cbp.tar.gz aua-inst-nix-cbp.tar.gz"
    else
        PKG_COMPONENT_FILE="app-common.tar.gz app-native-nix-others.tar.gz app-nix-cbp.tar.gz aua-common.tar.gz aua-native-nix-others.tar.gz aua-nix-cbp.tar.gz util-common.tar.gz util-nix-cbp.tar.gz properties-common.tar.gz app-inst-nix-cbp.tar.gz aua-inst-nix-cbp.tar.gz"
    fi
    if [ -z "$PKG_COMPONENT_FILE" ]; then
        echo "No component package is defined"
    else
        for word in $PKG_COMPONENT_FILE
        do
            verifyAndSetupComponent SETUP_RESULT "$PKG_DIR" "$word" "$APP_TMP" "others"
            if [ "$SETUP_RESULT" = "N" ] ; then
                echo "Failed to prepare component files."
                return
            fi
        done
    fi

    rm -rf "$PKG_DIR"
    eval $PREPARE_RESULT="Y"
}

# Function to get string segment with seperator '='
# e.g. $1 = "APP_HOME=/usr/local/obm APP_HOME=/root/Documents/obm" , $2 = 3
# then this function returns /root/Documents/obm
getColumnValue()
{
    STRING_PATTERN="$1"
    COL_NUM=$2
    echo $STRING_PATTERN | awk -F"=" '{print $'"${COL_NUM}"'}'
}

# Backup user setting files
# 1 - Backup Folder
backupUserConfig()
{
    TEMP_DIR="$1"
    BACKUP_DIR="$APP_HOME/$TEMP_DIR"

    # Get INIT_PATH to get APP_HOME from running service
    case "`uname`" in
        Linux*)      INSTALL_TYPE=NIX ;;
        Solaris*)    INSTALL_TYPE=NIX ;;
        SunOS*)      INSTALL_TYPE=NIX ;;
        OpenBSD*)    INSTALL_TYPE=OPENBSD ;;
        *BSD*)       INSTALL_TYPE=BSD ;;
        **)          INSTALL_TYPE=NOT_SUPPORTED ;;
    esac

    # Initialize INIT_PATH to empty
    INIT_PATH=""
    # Set INIT_PATH for different OS
    if [ $INSTALL_TYPE = "NIX" ]; then
        if [ -z "$INIT_PATH" ]; then
            if [ -d /etc/init.d ]; then
                INIT_PATH=/etc/init.d
            elif [ -d /etc/rc.d/init.d ]; then
                INIT_PATH=/etc/rc.d/init.d
            fi
        fi

    elif [ $INSTALL_TYPE = "BSD" ]; then
        INIT_PATH=/usr/local/etc/rc.d

    elif [ $INSTALL_TYPE = "OPENBSD" ]; then
        # get OpenBSD version
        OPENBSD_VERSION=`uname -r | cut -f1 -d"."`

        if [ $OPENBSD_VERSION -ge 5 ]
        then
            INIT_PATH=/etc/rc.d
        else
            INIT_PATH=/etc
        fi
    fi

    # Get old APP_HOME
    if [ -f "$APP_HOME/bin/uninstall.sh" ]; then
        APP_HOME_OLD=$APP_HOME
    elif [ ! -z "$INIT_PATH" ]; then
        OBM_SCRIPT_FILE="$INIT_PATH/cbpscheduler"
        if [ -f $OBM_SCRIPT_FILE ] ; then
            OBM_HOME_INFO=`cat $OBM_SCRIPT_FILE | grep APP_HOME=`
            APP_HOME_OLD=`getColumnValue "$OBM_HOME_INFO" 3`
        fi
    fi

    if [ -z "$APP_HOME_OLD" ]; then
        echo "  No old application found, begin fresh install"
    else
        if [ -f "$APP_HOME_OLD/bin/uninstall.sh" ]; then
            echo "  Backup user setting files"

            # directory for temporary keeping backup files
            if [ -d "$BACKUP_DIR" ]; then
                echo "  Remove previous cache"
                rm -rf "$BACKUP_DIR"
            fi
            mkdir "$BACKUP_DIR"

            # directory to be backup / restore
            DIR_NAME="log"
            for DIR in $DIR_NAME; do
                if [ -d "$APP_HOME_OLD/$DIR" ]; then
                    echo "    Backup $DIR"
                    mv -f "$APP_HOME_OLD/$DIR" "$BACKUP_DIR/$DIR"
                fi
            done

            # file to be backup / restore
            FILE_NAME="home.txt config.ini afc.opt cb.opt"
            for FILE in $FILE_NAME; do
                if [ -f "$APP_HOME_OLD/$FILE" ]; then
                    echo "    Backup $FILE"
                    mv -f "$APP_HOME_OLD/$FILE" "$BACKUP_DIR/$FILE"
                fi
            done
            echo "  Backup finished"

            # Run uninstall to stop Scheduler, AUA services
            echo "  Uninstall previous version..."
            "$APP_HOME_OLD/bin/uninstall.sh" "no_privilege_check"

            echo "  Remove previous application files"
            APP_DATA="`ls "$APP_HOME_OLD" | sort `"
            for DATA in $APP_DATA; do
                if [ "$DATA" != "$TEMP_DIR" ] ; then
                    if [ -d "$APP_HOME_OLD/$DATA" ] || [ -L "$APP_HOME_OLD/$DATA" ]; then
                        echo "    Remove directory $DATA"
                        rm -rf "$APP_HOME_OLD/$DATA"
                    else
                        if [ -z "`echo \"$APP_HOME_OLD/$DATA\" | grep ${INSTALLER}$`" ]; then
                            if [ -f "$APP_HOME_OLD/$DATA" ]; then
                                echo "    Remove file $DATA"
                                rm "$APP_HOME_OLD/$DATA"
                            fi
                        fi
                    fi
                fi
            done
            echo "  Remove application files finished"
        else
            echo "  No old application found, begin fresh install"
        fi
    fi
}

# Restore user data files
# 1 - Backup Directory
restoreUserConfig()
{
    # directory for temporary keeping backup files
    TEMP_DIR="$1"
    BACKUP_DIR="$APP_HOME/$TEMP_DIR"

    if [ -d "$BACKUP_DIR" ]; then
        echo "  Restore Previous Setting backup..."

        # directory to be backup / restore
        DIR_NAME="log"
        for DIR in $DIR_NAME; do
            if [ -d "$BACKUP_DIR/$DIR" ]; then
                echo "    Restore $DIR"
                mv -f "$BACKUP_DIR/$DIR" "$APP_HOME/$DIR"
            fi
        done

        # migration logic for properties file 'afc.opt' and 'cb.opt'.
        FILEPATH_UTIL="$APP_HOME/util/lib/installer-util.jar"
        CLASSNAME_UTIL=PropertiesMigration

        # file to be backup / restore
        FILE_NAME="home.txt config.ini afc.opt cb.opt"
        for FILE in $FILE_NAME; do
            if [ -f "$BACKUP_DIR/$FILE" ]; then
                JVM_EXEC="$JVM_HOME/bin/java"
                # afc.opt AND cb.opt need perform migration
                if [ "$FILE" = "afc.opt" ]; then
                    echo "    Restore application opt file (afc.opt)"
                    FILEPATH_OUTPUT="$BACKUP_DIR/afc.opt.new"
                    FILEPATH_DEFAULT="$BACKUP_DIR/afc.opt"
                    FILEPATH_CUSTOM="$APP_HOME/afc.opt"
                    "${JVM_EXEC}" -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                    mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
                else
                    if [ "$FILE" = "cb.opt" ]; then
                        echo "    Restore application opt file (cb.opt)"
                        FILEPATH_OUTPUT="$BACKUP_DIR/cb.opt.new"
                        FILEPATH_DEFAULT="$BACKUP_DIR/cb.opt"
                        FILEPATH_CUSTOM="$APP_HOME/cb.opt"
                        "${JVM_EXEC}" -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                        mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
                    else
                        echo "    Restore $FILE"
                        mv -f "$BACKUP_DIR/$FILE" "$APP_HOME/$FILE"
                    fi
                fi
            fi
        done

        rm -rf "$BACKUP_DIR"
        echo "  Previous Setting backup restored"
    fi
}

# ---------------------------------
# Initialize
cd `dirname "$0"`

INSTALLER="$0"
MODE="manual"
if [ ! -z "$1" ]; then
    if [ "$1" = "rpm" ]; then
        MODE="rpm"
    elif [ "$1" = "deb" ]; then
        MODE="deb"
    fi
fi
APP_HOME=`pwd`
if [ ! -z "$2" ]; then
    APP_HOME="$2"
fi
PRIVILEGE_CHK=""
if [ ! -z "$3" ]; then
    PRIVILEGE_CHK="$3"
fi
APP_BIN=$APP_HOME/bin
TIMESTAMP_DIR=`date '+%y%m%d%H%M%S'`
APP_TMP="/tmp/_cbp.$TIMESTAMP_DIR"
PKG_DIR="$APP_HOME/app.pkg"
TEMP_CONFIG_DIR="_oldcfg"

# get arch of this system
MY_PLATFORM=""
case "`uname`" in
    Solaris* | SunOS*)
        MY_PLATFORM="solaris"
        case "`isainfo -b`" in
            32 | 64)
                MY_ARCH="32"
                ;;
            # amd64 is NOT support in Solaris / SunOS
            #64)
            #    MY_ARCH="64"
            #    ;;
            *)
                MY_ARCH="Others"
                ;;
        esac
        ;;
    *)
        MY_PLATFORM="others"
        case "`uname -m`" in
            i[3-6]86)
                MY_ARCH="32"
                ;;
            x86_64 | amd64)
                MY_ARCH="64"
                ;;
            *)
                MY_ARCH="Others"
                ;;
        esac
        ;;
esac

# Print Logging Message Header
echo "Log Time: `date`"

# Prepare packaged components
if [ -d "$APP_TMP" ]; then
    rm -rf "$APP_TMP"
fi
mkdir "$APP_TMP"

preparePackagedComponents RESULT JVM_HOME_RESULT
if [ "$RESULT" = "N" ] ; then
    echo "Error in preparing files, installer will terminate"
    rm -rf "$APP_TMP"
    exit 1
fi

# Backup user setting files
backupUserConfig "$TEMP_CONFIG_DIR"

# Move packaged components
if [ ! -d "$APP_HOME" ]; then
    mkdir "$APP_HOME"
fi
mv "$APP_TMP"/* "$APP_HOME"
rm -rf "$APP_TMP"

# Verify if JVM home exist
cd "$APP_HOME"
JVM_HOME=$APP_HOME/jvm
if [ "$JVM_HOME_RESULT" = "N" ]; then
    USER_JVM_HOME=""
    echo "Please enter your java 1.7 home: "
    read USER_JVM_HOME
    if [ -z "$USER_JVM_HOME" ]; then
        echo "Invalid java home!"
        exit 1
    fi
    if [ ! -d "$USER_JVM_HOME" ]; then
        echo "$USER_JVM_HOME does not exist!"
        exit 1
    fi
    JVM_SECURITY="$APP_HOME/util/jvm/lib/security"
    if [ -d "$JVM_SECURITY" ]; then
        echo "Copy java 1.7 from $USER_JVM_HOME"
            if [ ! -d "$JVM_HOME" ]; then
                cp -rf "$USER_JVM_HOME" "$JVM_HOME"
            else
                cp -rf $USER_JVM_HOME/* $JVM_HOME
            fi
        cp -rf $JVM_SECURITY/* $JVM_HOME/lib/security
    else
        echo "Create symbolic link from $USER_JVM_HOME"
        ln -sf "$USER_JVM_HOME" "$JVM_HOME"
    fi
fi

# Install Application
TIMESTAMP_LOG=`date '+%y%m%d%H%M%S'`
INSTALL_LOG="$APP_HOME/install-$TIMESTAMP_LOG.log"
INSTALL_SCRIPT="$APP_BIN/install.sh"
if [ ! -f "$INSTALL_SCRIPT" ]; then
    echo "Install script not exist"
    exit 1
fi
echo "Install Application Path: $APP_HOME"
sh "$INSTALL_SCRIPT" "$PRIVILEGE_CHK" >> "$INSTALL_LOG"

# Restore user data files
restoreUserConfig "$TEMP_CONFIG_DIR"

# Exit
echo "Done"
exit 0
